import {
  scaleLinear,   // for the linear scales
  scaleOrdinal,  // for the ordinal scales
  extent,  // to get the min/max values in a dataset
  axisLeft,  // to create  y-axis
  axisBottom,  // to create an x-axis
  symbols,  // array of symbol types
  symbol,  // for generating the  symbols
} from 'd3';

// declaring a scatter plot function as a reusable component
export const scatterPlot = () => {
  let xValue;
  let yValue;let width;
  let height;
  let data;
  let symbolValue;
  let margin;
  let size;

  const my = (selection) => {
    
    // Creating linear scales for x and y axis
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);
    
		// Creating an ordinal scale to map data symbols
    const symbolScale = scaleOrdinal()
      .domain(data.map(symbolValue))
      .range(symbols);
    
    // Creating an ordinal scale to map data colors
    const colorScale = scaleOrdinal()
    .domain(data.map(symbolValue))
    .range(['orange', 'violet', 'seagreen']); // Add colors here

    const symbolGenerator = symbol().size(size);
    
    // Creating an array of objects with x, y coordinates and symbol paths for each data point
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      color: colorScale(symbolValue(d)),  // Add color here
      species: symbolValue(d),  // Add species name here
      pathD: symbolGenerator.type(symbolScale(symbolValue(d)))()
    }));

    selection
      .selectAll('path')
      .data(marks)
      .join('path')
      .attr('d', (d) => d.pathD)
      .attr('transform', (d) => `translate(${d.x},${d.y})`)
    	.attr('fill', (d) => d.color); // Set fill color here;

    // Creating a y-axis
  selection
    .append('g')
    .attr('transform', `translate(${margin.left},0)`)
    .call(axisLeft(y));

   // Creating an x-axis
  selection
    .append('g')
    .attr('transform', `translate(0,${height - margin.bottom})`)
    .call(axisBottom(x));
  
 

   // Append a new "g" element to the current selection, and add an axis to it.
   selection
      .append('g')
      .attr('transform', `translate(${margin.left},0)`)
      .call(axisLeft(y));
   
    // Append a new "g" element to the current selection, and add an axis to it.
    selection
      .append('g')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x));
  };

  
	 // Define a function "my" that can be used to configure the chart.
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.symbolValue = function (_) {
    return arguments.length
      ? ((symbolValue = _), my)
      : symbolValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.size = function (_) {
    return arguments.length ? ((size = +_), my) : size;
  };

  return my;
};
